syntax = "proto3";

package mwitkow.testproto;

option go_package = "gitlab.com/gitlab-org/gitaly/v14/internal/praefect/grpc-proxy/testdata";

message Empty {
}

message PingRequest {
  string value = 1;
}

message PingResponse {
  string Value = 1;
  int32 counter = 2;
}

service TestService {
  rpc PingEmpty(Empty) returns (PingResponse) {}

  rpc Ping(PingRequest) returns (PingResponse) {}

  rpc PingError(PingRequest) returns (Empty) {}

  rpc PingList(PingRequest) returns (stream PingResponse) {}

  rpc PingStream(stream PingRequest) returns (stream PingResponse) {}

}

