# This file is generated by generate-proto-ruby. Do not edit.
$:.unshift(File.expand_path('../gitaly', __FILE__))

require 'gitaly/version'

require 'gitaly/blob_services_pb'

require 'gitaly/cleanup_services_pb'

require 'gitaly/commit_services_pb'

require 'gitaly/conflicts_services_pb'

require 'gitaly/diff_services_pb'

require 'gitaly/hook_services_pb'

require 'gitaly/internal_services_pb'

require 'gitaly/namespace_services_pb'

require 'gitaly/objectpool_services_pb'

require 'gitaly/operations_services_pb'

require 'gitaly/praefect_services_pb'

require 'gitaly/ref_services_pb'

require 'gitaly/remote_services_pb'

require 'gitaly/repository-service_services_pb'

require 'gitaly/server_services_pb'

require 'gitaly/smarthttp_services_pb'

require 'gitaly/ssh_services_pb'

require 'gitaly/transaction_services_pb'

require 'gitaly/wiki_services_pb'

