package gitalypb

// Code generated by protoc-gen-gitaly. DO NOT EDIT

// GitalyProtos is a list of gitaly protobuf files
var GitalyProtos = []string{
	"blob.proto",
	"cleanup.proto",
	"commit.proto",
	"conflicts.proto",
	"diff.proto",
	"errors.proto",
	"hook.proto",
	"internal.proto",
	"lint.proto",
	"namespace.proto",
	"objectpool.proto",
	"operations.proto",
	"praefect.proto",
	"ref.proto",
	"remote.proto",
	"repository-service.proto",
	"server.proto",
	"shared.proto",
	"smarthttp.proto",
	"ssh.proto",
	"transaction.proto",
	"wiki.proto",
}
