syntax = "proto3";

package test.chunk;

option go_package = "gitlab.com/gitlab-org/gitaly/v14/internal/helper/chunk/testdata";

service Test {
  rpc StreamOutput(StreamOutputRequest) returns (stream StreamOutputResponse) {}
}

message StreamOutputRequest {
  int32 bytes_to_return = 1;
}
message StreamOutputResponse {
  repeated bytes msg = 1;
}
