package x509

// SSLCertDir and SSLCertFile are the environment variables that specify the
// location to add system root certificates.
const (
	SSLCertDir  = "SSL_CERT_DIR"
	SSLCertFile = "SSL_CERT_FILE"
)
